use serde::{Serialize, Deserialize};
use std::fs::File;
use std::str::FromStr;
use std::io::prelude::*;
use std::collections::HashMap;

static FILE_CONF: &str = "mrprox.conf";
static FILE_SERVERS: &str = "mrprox_servers.conf";
const DEF_PORT: u16 = 25565;

#[derive(Debug, PartialEq, Serialize, Deserialize, Clone)]
pub struct ConfFile{
    port: String,
    port_conf: String,
    conf_type: String,
}

#[derive(Debug, PartialEq, Serialize, Deserialize, Clone)]
pub struct ServerData{
    pub domain: String,
    pub ip: String,
}

pub struct Config{
    l_servers : HashMap<String, String>,
    port: String,
    port_conf: String,
    conf_type: String,
}

impl Config {
    pub fn new() -> Self {
        let mut conf_file = File::open(&FILE_CONF).unwrap();
        let mut servers_file = File::open(&FILE_SERVERS).unwrap();
        let mut s1 = String::new();
        let mut s2 = String::new();
        conf_file.read_to_string(&mut s1).unwrap();
        servers_file.read_to_string(&mut s2).unwrap();
        let yam_conf: ConfFile = serde_yaml::from_str(&s1).unwrap();
        let yam_ser: Vec<ServerData> = serde_yaml::from_str(&s2).unwrap();
        Self{
            l_servers: Self::get_servers(&yam_ser),
            port: yam_conf.port,
            port_conf: yam_conf.port_conf,
            conf_type: yam_conf.conf_type,
        }
    }

    pub fn add(&mut self, server: ServerData) -> bool{
        match self.l_servers.insert(server.domain, server.ip){
            Some(_s) => true, //Modified instead of added
            None => false,
        }
    }

    pub fn del(&mut self, domain: String) -> bool {
        match self.l_servers.remove(&domain) {
            Some(_s) => true,
            None => false,
        }
    }

    pub fn flush(&self){
        let servers = Vec::from_iter(self.l_servers.iter()
                    .map(|(x, y)| ServerData{domain: (*x).clone(), ip: (*y).clone()}));
        let mut file = File::create(FILE_SERVERS).unwrap();
        file.write(&serde_yaml::to_string(&servers).unwrap().into_bytes());
        file.flush();

    }

    fn get_servers(file: &Vec<ServerData>) -> HashMap<String, String> {
        let mut ret = HashMap::new();
        for j in file{
            ret.insert(j.domain.clone(),j.ip.clone());
        }
        ret
    }

    pub fn get_host(&self, host: &String) -> Option<(&String, &String)>{
        self.l_servers.get_key_value(host)
    }

    pub fn get_server(&self, host: &String) -> Option<(String, u16)>{
        let raw: Vec<&str> =
        match self.get_host(host){
            Some(h) => h.1.split(":").collect(),
            None => return None,
        };

        match raw.len() {
            1 => Some((String::from(raw[0]),DEF_PORT)),
            2 => {
                match FromStr::from_str(raw[1]) {
                    Ok(p) => Some((String::from(raw[0]),p)),
                    Err(_e) => None,
                }
            },
            _=> None,
        }
    }

    pub fn get_port(&self) -> &String{
        &self.port
    }

    pub fn get_port_conf(&self) -> &String{
        &self.port_conf
    }

    pub fn get_conf_type(&self) -> &String{
        &self.conf_type
    }
}

